#!/usr/bin/env python3

# Check links to images.

#from git import Repo
from dulwich.repo import Repo
from dulwich import diff_tree
from dulwich.patch import write_tree_diff
import dulwich.contrib.diffstat
import difflib
import markdown
import os
import re
import sys
import tomllib
import io

def get_link(url):
    link = url.group("link")

    if not link[:8] == "/images/":
        return

    if not link[8:] in links:
        links.append(link[8:])


def parse_file(filename):
    required_headers = ["date", "title", "foo"] 

    # print("%s: " % filename)
    with open(filename, "r+") as file:
        contents = file.read()

        if not contents[0:3] == "+++":
            return

        fm = contents.split("+++")[1]
        doc = contents.split("+++", 2)[2]

        headers = tomllib.loads(fm)

        for required in required_headers:
            if not required in headers:
                errors.append(f"ERROR: Missing \"{ required }\" header in { filename.decode('ascii') }")
        
        return

        # Check the header for the required         
        # Convert to HTML to simplify link detection
        text = markdown.markdown(doc)

        # From img tags
        pattern = re.compile("src=\"(?P<link>.*?)\"")
        doc = pattern.sub(get_link, text)

        # We have some links to bits in images
        pattern = re.compile("href=\"(?P<link>.*?)\"")
        doc = pattern.sub(get_link, text)


images = []

# Access git and list changed files
repo = Repo(sys.argv[1])

# If we are in a merge commit, we can get the BASE_SHA, so we can do something like:
# FILES=$(git diff --name-only $CI_MERGE_REQUEST_DIFF_BASE_SHA...HEAD)
# to get a list of all the changed files. When we are running on a progress branch
# (without a MR) we'll have to assume a delta against the master branch?
BASE_SHA = os.environ.get('CI_MERGE_REQUEST_DIFF_BASE_SHA')

TARGET_SHA = os.environ.get('CI_MERGE_REQUEST_TARGET_BRANCH_SHA')
COMMIT_SHA = os.environ.get('CI_COMMIT_SHA')

print("BASE_SHA:%s\n" % BASE_SHA)
print("TARGET_SHA:%s\n" % TARGET_SHA)
print("COMMIT_SHA:%s\n" % COMMIT_SHA)

#head = repo[TARGET_SHA]
#diff = head.diff(COMMIT_SHA)

head = repo.head()
base = repo.get_parents(repo.head())[0]

changes = diff_tree.tree_changes(repo, repo[head].tree, repo[base].tree)

files = []
for change in changes:
    if change.type == "modify" or change.type == "add":
        outstream = io.BytesIO()
        write_tree_diff(outstream, repo.object_store, repo[base].tree, repo[head].tree)
        string  = outstream.getvalue()
        print(string)
        stat = dulwich.contrib.diffstat.diffstat(string)
        print(stat)
        if not change.new.path in files:
            files.append(change.new.path)

errors = []

for file in files:
    parse_file(file)

if len(errors) == 0:
    sys.exit(0)

for error in errors:
    print(error)

sys.exit(1)

links = []

broken = []
for link in links:
    sys.stdout.flush()

    if link in images:
        images.remove(link)
    else:
        broken.append(link)

print(f"Found {len(broken)} broken image links in {len(links)} tested:")
broken.sort()
for b in broken:
    print(" ", b)

print(f"Found {len(images)} unused images:")
images.sort()
for i in images:
    print(" ", i)

sys.exit(1 if broken or images else 0)
